//////////////////////////////////////////////////////////////////////
//
// client.cpp
//
// client class
//
//
//                                                    XIII
//////////////////////////////////////////////////////////////////////

#ifndef _CLIENT_CPP_
#define _CLIENT_CPP_

#include "client.h"

#include "lib.h"

#include "md5sum.h"

CLIENT::CLIENT():
IP(NULL),
hash(NULL),
hash_type(0),
password(NULL),
border(NULL),
state(Free),
borderstate(First)
{

}

CLIENT::CLIENT(char *_IP)
{
	set_IP(_IP);
	hash=NULL;
	hash_type=0;
	password=NULL;
	border=NULL;
	state=Free;
	borderstate=First;
}

CLIENT::CLIENT(const CLIENT &rhs)
{	
	//
	// Copy Constructor
	//
	if(this==&rhs)
		return;

	IP=NULL;
	hash=NULL;
	hash_type=0;
	password=NULL;
	border=NULL;
	state=Free;
	borderstate=First;
	set_IP(rhs.get_IP());
	set_alphabet(rhs.get_alphabet());
	set_hash(rhs.get_hash());
	set_hash_type(rhs.get_hash_type());
	set_password(rhs.get_password());
	set_border(rhs.get_border());
}

CLIENT::~CLIENT()
{
	if(IP)
	{
		delete [] IP;
		IP=NULL;
	}
	if(hash)
	{
		delete []hash;
		hash=NULL;
	}
	if(password)
	{
		delete []password;
		password=NULL;
	}
	if(border)
	{
		delete []border;
		border=NULL;
	}
}

void CLIENT::set_IP(char* _IP)
{
	if(_IP)
	{
		int len = strlen(_IP);
		if(IP)
		{
			delete [] IP;
			IP=NULL;
		}
		IP = new char [len+1];
		if(!IP)
		{
			prerr("Cant allocate memory for IP");
			return;
		}
		ZeroMemory(IP,len+1);
		strcpy(IP,_IP);
	}
}

char* CLIENT::get_IP() const
{
	return IP;
}

void CLIENT::set_alphabet(char* _alphabet)
{
	alphabet.set_alphabet(_alphabet);
}

char* CLIENT::get_alphabet() const
{
	return alphabet.get_alphabet();
}

void CLIENT::set_hash(char* _hash)
{
	if(_hash)
	{
		int len = strlen(_hash);
		if(hash)
		{
			delete [] hash;
			hash=NULL;
		}
		hash = new char [len+1];
		if(!hash)
		{
			prerr("Cant allocate memory for hash");
			return;
		}
		ZeroMemory(hash,len+1);
		strcpy(hash,_hash);
	}
}

char* CLIENT::get_hash() const
{
	return hash;
}

void CLIENT::set_hash_type(int _hash_type)
{
	hash_type = _hash_type;
}

int CLIENT::get_hash_type() const
{
	return hash_type;
}

void CLIENT::set_password(char* _password)
{
	if(_password)
	{
		int len = strlen(_password);
		if(password)
		{
			delete [] password;
			password=NULL;
		}
		password = new char [len+1];
		if(!password)
		{
			prerr("Cant allocate memory for password");
			return;
		}
		ZeroMemory(password,len+1);
		strcpy(password,_password);
	}
}

char* CLIENT::get_password() const
{
	return password;
}

void CLIENT::set_border(char* _border)
{
	if(_border)
	{
		int len = strlen(_border);
		if(border)
		{
			delete [] border;
			border=NULL;
		}
		border = new char [len+1];
		if(!border)
		{
			prerr("Cant allocate memory for border");
			return;
		}
		ZeroMemory(border,len+1);
		strcpy(border,_border);
	}
}

char* CLIENT::get_border() const
{
	return border;
}

void CLIENT::set_state(STATE _state)
{
	state=_state;
}

STATE CLIENT::get_state() const
{
	return state;
}

void CLIENT::set_borderstate(BORDERSTATE _borderstate)
{
	borderstate=_borderstate;
}

BORDERSTATE CLIENT::get_borderstate() const
{
	return borderstate;
}

void CLIENT::crack()
{
	if(!password)
	{
		prerr("define password first!");
		return;
	}
	if(!border)
	{
		prerr("define border first!");
		return;
	}
	int pl=strlen(password);
	int bl=strlen(border);
	if(pl==0 || bl==0 || pl>bl)
	{
		prerr("bad password(%s) or border(%s)",password,border);
		return;
	}
	if(!normalize_hash())
		return;
	state = Busy;
	bruteforce();
}

bool CLIENT::normalize_hash()
{
	//
	// normalize hash to hash_type
	//
	
	if(!hash)
		return false;

	char* ct=NULL;
	int i,j,it=0;

	switch(hash_type)
	{
	case 0:
		//
		// md5
		//
		//break;
	default:
		//
		//# md5 by default :)
		//
		{
			it=strlen(hash);
			if(it>TEXT_MD5_BUF_SIZE)
			{
				prerr("MD5 hash must contain %d symbols",TEXT_MD5_BUF_SIZE);
				for(i=TEXT_MD5_BUF_SIZE;i<it;i++)
					hash[i]='\0';
			}
			if(it<TEXT_MD5_BUF_SIZE)
			{	
				//
				// add zero from left to get normal hash-length
				//
				ct = new char [TEXT_MD5_BUF_SIZE+1];
				if(!ct)
				{
					prerr("Cant allocate memory for normalized hash!");
					return false;
				}
				ZeroMemory(ct,TEXT_MD5_BUF_SIZE+1);
				i=TEXT_MD5_BUF_SIZE-it;
				for(j=0;j<i;j++)
				{
					ct[j]='0';
				}
				strcpy(&ct[i],hash);
				delete []hash;
				hash=ct;
			}
#ifdef DEBUG
			printf("[H] Hash: %s\n",hash);
#endif // #ifdef DEBUG
		}
		break;
	}
	return true;
}

int CLIENT::bruteforce()
{
	int i,j;
	char* alph = alphabet.get_alphabet();
again:
	int alen= alphabet.length();
	int plen=strlen(password);
	int blen=strlen(border);
	char ct;

	j=plen-1;
	i=j;
	do
	{
		if(state==Free)
			return 4;
		if(check_password(plen))
		{
			state=Victory;
			break;
		}

		if( !check_border(plen,blen) )
		{
			break;
		}

		i=j;
		if(j>=0)
		{
			ct=alphabet.get_next_symbol(password[j]);
			if( ct!=1)
			{
				password[j]=ct;
				continue;
			}
			else
			{
				password[j]=alph[0];

				if(i==0)
					goto check;
				
				while(i>0)
				{
					i--;
					ct=alphabet.get_next_symbol(password[i]);
					if(ct!=1)
					{
						password[i]=ct;
						break;
					}
					else
					{
						password[i]=alph[0];
						if(i==0)
							goto check;
						else
							continue;
					}
				}	
			}
		}
	}
	while(check_border(plen,blen));

check:
	if(state==Victory)
		return 1;
	if(state!=Bordered)
	{
		if(blen>plen)
		{
			char* st=new char[plen+2];
			if(!st)
			{
				prerr("cant allocate memory for password");
				return -1;
			}
			ZeroMemory(st,plen+2);
			strcat(st," ");
			strcat(st,password);
			st[0]=alph[0];
			delete []password;
			password = st;
			st=NULL;
			goto again;
		}
	}
	return 0;
}

int CLIENT::check_password(int _passlength)
{
	switch(hash_type)
	{
	case 0:
		//
		// md5
		//
		//break;
	default:
		//
		//# md5 by default :)
		//
		{
			unsigned char ubuf[MD5_BUF_SIZE];
			char wbuf[TEXT_MD5_BUF_SIZE+1], wt[10];
			ZeroMemory(ubuf,MD5_BUF_SIZE);
			ZeroMemory(wbuf,MD5_BUF_SIZE*2+1);
			ZeroMemory(wt,10);
			md5_buffer(password,_passlength,ubuf);
			for(int i=0;i<MD5_BUF_SIZE;i++)
			{
				wsprintf(wt,"%02x",ubuf[i]);
				strcat(wbuf, wt);
			}
			//-
#ifdef HASHLOG
			writehash(password,wbuf);
#endif //#ifdef HASHLOG
			//-
			if(!strcmp(wbuf,hash))
				return 1;
			else
				return 0;
		}
		break;
	}
	return 0;
}

int CLIENT::check_border(int _passlength,int _borderlength)
{
	if(_passlength>_borderlength)
		return 0;

	if(!strcmp(password,border))
	{
		state=Bordered;
		return 0;
	}
	else
		return 1;

	return 0;
}

#endif // #ifndef _CLIENT_CPP_
